/*
 * Program:	Main stand-alone Pine Composer routines
 *
 * Modifier:	Michael Seibel
 *		Networks and Distributed Computing
 *		Computing & Communications
 *		University of Washington
 *		Administration Building, AG-44
 *		Seattle, WA  98195
 *		Internet: mikes@cac.washington.edu
 *
 * Date:	Nov 1989
 * Last Edited:	9 September 1991
 *
 * Copyright 1991 by the University of Washington
 *
 *  Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies and that both the
 * above copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the University of Washington not be
 * used in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  This software is made
 * available "as is", and
 * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
 * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *
 * WEEMACS/PICO NOTES:
 *
 * 08 Jan 92 - removed PINE defines to simplify compiling
 *
 */

#include        <stdio.h>
#include	<setjmp.h>
#include	"osdep.h"	/* operating system dependent includes */
#include        "estruct.h"	/* global structures and defines */
#include	"efunc.h"	/* function declarations and sans name table */
#include	"edef.h"	/* global definitions */
#include	"pico.h"	/* weemacs composer definitions */


#ifndef GOOD
#define GOOD    0
#endif

/*
 * this isn't defined in the library, because it's a pine global
 * which we use for GetKey's timeout
 */
long	timeout = 0L;			/* global timeout value		*/


/*
 * main standalone pico routine
 */
main(argc, argv)
char    *argv[];
{
        register int    c;
        register int    f;
        register int    n;
	register BUFFER *bp;
	register int	carg;		/* current arg to scan */
	extern   int    new_mail();
	int	 viewflag = FALSE;	/* are we starting in view mode? */
	int	 i;
        char	 bname[NBUFN];		/* buffer name of file to read */

	/* initialize the editor and process the startup file */

	for_pine = 0;			/* turn OFF composer functionality */

	/*
	 * Read command line flags before initializing, otherwise, we never
	 * know to init for f_keys...
	 */
	carg = 1;
	while(carg < argc){
	    if(argv[carg][0] == '-'){
		switch(argv[carg][1]){
		  case 'v':	/* -v for View File */
		  case 'V':
		    viewflag = !viewflag;
		    break;
		  case 'f':	/* -f for function key use */
		    use_fkeys = !use_fkeys;
		    break;
		  case 'n':	/* -n for new mail notification */
		    timeout = 180L;
		    if(argv[carg][2] != '\0')
		      if((timeout = (long)atoi(&argv[carg][2])) < 30)
			timeout = 180L;
		    break;
		  case 'z':	/* -z to suspend */
		    allow_suspend = !allow_suspend;
		    break;
		  case 'w':	/* -w turn off word wrap */
		    gmode ^= MDWRAP;
		    break;
		  default:	/* unknown switch */
		    /* ignore this for now */
		    break;
		}
		carg++;
	    }
	    else			/* pick up file name later... */
	      break;
	}

        vtinit();			/* Displays.            */
        strcpy(bname, "main");		/* default buffer name */
        edinit(bname);			/* Buffers, windows.   */

	update();			/* let the user know we are here */

#if	TERMCAP
	if(kpadseqs == NULL){	/* let user know if the arrow keys work */
	    (*term.t_putchar)('\007');
	    emlwrite("Warning: keypad keys may non-functional");
	}
#endif	/* TERMCAP */

	/*
	 * Any file to edit?
	 */
	if(carg < argc){
	    /* set up a buffer for this file */
	    makename(bname, argv[carg]);

	    /* if this is the first file, read it in */
		bp = curbp;
		makename(bname, argv[carg]);
		strcpy(bp->b_bname, bname);
		strcpy(bp->b_fname, argv[carg]);
		if (readin(argv[carg], (viewflag==FALSE))
		    == ABORT) {
		    strcpy(bp->b_bname, "main");
		    strcpy(bp->b_fname, "");
		}
		bp->b_dotp = bp->b_linep;
		bp->b_doto = 0;

	    if (viewflag)			/* set the view mode */
	      bp->b_mode |= MDVIEW;
	}

	/* setup to process commands */
        lastflag = 0;                           /* Fake last flags.     */
	curbp->b_mode |= gmode;			/* and set default modes*/

	curwp->w_flag |= WFMODE;		/* and force an update	*/

	if(timeout)
	  emlwrite("Checking for new mail every %D seconds", timeout);

	while(1){

	    update();				/* Fix up the screen    */

	    c = GetKey();	

	    if(c == NODATA || time_to_check()){
		if(new_mail())
		  emlwrite("You may possibly have new mail.");

		if(c == NODATA)
		  continue;
	    }

	    if (mpresf != FALSE) {
		if (mpresf++ > MESSDELAY)
		  mlerase();
	    }

	    f = FALSE;
	    n = 1;

	    execute(c, f, n);			/* Do it.               */
	}
}




/*
 * Stubs of routines defined in PINE source code (which isn't
 * compiled with the stand alone version)...
 */

char h_composer_ins[] = "";			/* various help text strings */
char h_composer_search[] = "";
char h_composer_to[] = "";
char h_composer_cc[] = "";
char h_composer_bcc[] = "";
char h_composer_fcc[] = "";
char h_composer_subject[] = "";
char h_composer[] = "";


/*
 * new_mail - pine's new_mail checking routine.
 */
int new_mail(i, j, k)
int *i, j, k;
{
    return(report_new_mail());
}



/*
 * helper - display the given help text
 */
helper(text, title, i)
     char *text[], *title;
     int i;
{
    register    int numline = 0;
    char        **p;
 
    p = text;
    while(*p++ != NULL) 
      numline++;
    return(wscrollw(COMPOSER_TOP_LINE, term.t_nrow-1, text, numline));
}


char *fnexpand(f, l)
char *f;
int   l;
{
    extern char *pfnexpand();

    return(pfnexpand(f, l));
}


/*
 * display_message - pine's error message poster
 */
display_message(c)
int  c;
{
    return;
}


/*
 * Raw - sets and unsets terminal modes and signal handlers in pine
 */
Raw(i)
int  i;
{
    return;
}


/*
 * init_keyboard - pine's func to initialize special keyboard functionality
 *                 in UW-enhanced NCSA telnet.
 */
init_keyboard(state)
int	state;
{
    if(state)
      printf("\033[99h");			/* turn on function keys */
    else
      printf("\033[99l");			/* turn of function keys */
}


/*
 * addr_book - pine's address book handling routine
 */
char *addr_book(i)
int  i;
{
    return(NULL);
}


/*
 * folders_for_fcc - pine provided interface to pine's mail folder handler
 */
folders_for_fcc(s)
char *s;
{
}


/*
 * build_address - pine's utility to expand and format email addresses
 */
build_address(x, y, z)
char  *x, *y, **z;
{
    return(FALSE);
}
